#ifndef __CGA_H__
#define __CGA_H__


class Bivector
{
	private :
		double a,b,c,d;

	public :

		Bivector();
		Bivector(double,double,double,double);
		~Bivector();

		double getE(int);
		void setE(int,double); 

		Bivector operator*(Bivector);

		void print(void);
};
typedef Bivector bv;

class Multivector
{
  protected:
		double a,b,c,d,e,f,g,h;
	public :

		Multivector();
		Multivector(double,double,double,double,double,double,double,double);
		~Multivector();

		void normalize(void);
		double getE(int);
		void setE(int,double);
		void print(void);		
};
typedef Multivector mv;

//basis Vectors,bivectors and trivectors
const mv E1(0,1,0,0,0,0,0,0);
const mv E2(0,0,1,0,0,0,0,0);
const mv E3(0,0,0,1,0,0,0,0);
const mv E12(0,0,0,1,0,0,0,0);
const mv E13(0,0,0,0,0,1,0,0);
const mv E23(0,0,0,0,0,0,1,0);
const mv E123(0,0,0,0,0,0,0,1);

//products
Multivector gp(Multivector,Multivector);//geometric product
Multivector op(Multivector,Multivector);//outer product
Multivector ip(Multivector,Multivector);// inner product
Multivector lcip(Multivector,Multivector);//left contraction inner product

Multivector operator*(Multivector ain,Multivector bin);//geometric product
Multivector operator/(Multivector ain,Multivector bin);//divide by multivector
Multivector operator^(Multivector,Multivector);//outer product

Multivector operator+(Multivector ain,Multivector bin);//add
Multivector operator*(Multivector ain,double s);//multiple by scalar
Multivector operator/(Multivector ain,double s);//divide by scalar
Multivector operator*(double s,Multivector ain);//multiple by scalar
Multivector operator/(double s,Multivector ain);//divide by scalar

//functions
Multivector operator-(Multivector,Multivector);//minus
Multivector operator!(Multivector);//inverse
Multivector operator~(Multivector);//reverse
Multivector operator-(Multivector ain);//negate


Multivector reverse(Multivector);
Multivector dual(Multivector);
Multivector conjugate(Multivector);
Multivector inverse(Multivector);

Multivector neg(Multivector); // sign change for all elements
double mag(Multivector ain);// magnitude

Multivector spinor(float,Multivector);// also know as a rotor
Multivector rotate(float,Multivector,Multivector);// rotations
Multivector translate(Multivector,Multivector,Multivector);//translations
#endif /*__CGA_H__*/

